/*
code for cddb query lend from cdcd - Command Driven CD player
Copyright (C) 1998-99 Tony Arcieri
Copyright (C) 2001 Fabrice BAUZAC
Licensed under GPL
*/

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>

#include "cddb.h"
#include "minterface.h"
#include "msupport.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
static void errorfunc(char *msg)
{
    GtkWidget *w;
    GladeXML *xml = create_error_dialog();

    w = glade_xml_get_widget(xml, "error_dialog_label");
    gtk_label_set_text(GTK_LABEL(w), msg);
}


void cleanstr(char *str)
{
    int i;
    for (i = 0; i < strlen(str); i++)
        if (str[i] == '_') str[i] = ' ';
    return;
}


/*****************************************************************************
 *
 * Functions to set album/artist/track data for a given CD
 *
 *****************************************************************************/
void cdda_cdinfo_cd_set(cdinfo_t * cdinfo, char *str)
{
    char *tok1, *tok2;

    tok1 = strtok(str, "/");
    tok2 = strtok(NULL,"/");

    cleanstr(tok1); cleanstr(tok2);

    cdinfo->albname = strdup(g_locale_to_utf8(tok2,strlen(tok2),NULL,NULL,NULL));
    cdinfo->artname = strdup(g_locale_to_utf8(tok1,strlen(tok1),NULL,NULL,NULL));
}

void cdda_cdinfo_track_set(cdinfo_t * cdinfo, gint num, gchar * artist, gchar * title)
{
    trackinfo_t *track = cdinfo->tracks + num;

    /* Check bounds */
    if (num < 0 || num >= 100)
        return;


    if (artist)
        cdinfo->tracks[num].artist = g_strdup(g_locale_to_utf8(artist,strlen(artist),NULL,NULL,NULL));
    else
        cdinfo->tracks[num].artist = NULL;

    cleanstr(title);
    cdinfo->tracks[num].title = g_strdup(g_locale_to_utf8(title,strlen(title),NULL,NULL,NULL));
    cdinfo->tracks[num].num = num;
}    

/*****************************************************************************
 *
 * Functions to query and read the CDDB server
 *
 *****************************************************************************/
static gchar *cddb_generate_hello_string(void)
{
    static gchar *buffer;
    buffer = g_strdup_printf("&hello=nobody+localhost+gtkcdlable+074");
    return buffer;
}
static gint cddb_http_open_connection(gchar * server, gint port)
{
    gint sock;
    gchar *status;

    if((sock = http_open_connection(server, 80)) == 0)
        status = "Failed";
    else
        status = "Ok";

    //printf("Connecting to CDDB-server %s: %s\n", server, status);
    return sock;
}

static char * cddb_generate_offset_string(cddb_toc_entry *cte)
{
    char *buffer;
    char tmp[128];
    int i;

    buffer = malloc(sizeof(char) * 4096); //this needs to be dynamic
    buffer[0] = '\0';

    for (i = 0; i < cte->numtrack; i++)
      {
        char tmp[128];
        sprintf(tmp,"%d",cte->offsets[i]);
        strcat(buffer,tmp);
        strcat(buffer,"+");
      }
    return buffer;
}

static gboolean cddb_query(gchar *server, cddb_toc_entry *cte, int proto, struct cddb_disc_header_t *info)
{
    /*
     * Query the cddb-server for the cd.
     * Returns the *real* diskid and category.
     */

    gint sock;
    gchar *offsets, *getstr;
    gchar buffer[256];
    gchar **response;
    gint i;
    long int discid;

    if((sock = cddb_http_open_connection(server, 80)) == 0)
        return FALSE;

    offsets = cddb_generate_offset_string(cte);
    discid = cte->discid;

    getstr = g_strdup_printf(
            "GET /~cddb/cddb.cgi?cmd=cddb+query+%08x+%d+%s%d+%s&proto=%d HTTP/1.0\r\n\r\n",
            discid, cte->numtrack,
            offsets, cte->length-2,
            cddb_generate_hello_string(), proto);

    //printf("\n\n%s\n", getstr);

    g_free(offsets);
    write(sock, getstr, strlen(getstr));
    g_free(getstr);

    if (http_read_first_line(sock, buffer, 256) < 0)
      {
        http_close_connection(sock);
        return FALSE;
      }

    http_close_connection(sock);
    response = g_strsplit(buffer, " ", 4);

    printf("Query response: %s\n", buffer);

    switch(strtol(response[0], NULL, 10))
      {
        case 200:
            /* One exact match */
            for (i = 0; i < 4; i++)
              {
                if (response[i] == NULL)
                  {
                    g_strfreev(response);
                    return FALSE;
                  }
              }
            info->category = g_strdup(response[1]);
            info->discid = strtoul(response[2], NULL, 16);
            break;
        default: /* FIXME: Handle other 2xx */
            g_strfreev(response);
            return FALSE;
      }

    g_strfreev(response);
    return TRUE;
}

static gboolean cddb_read(gchar *server, int proto, cddb_disc_header_t *cddb_info, cdinfo_t *cdinfo)
{
    gint sock;
    gchar *readstr;
    gchar buffer[256];
    gchar *realstr, *temp;
    gint len, command, bufs;
    gint num, oldnum;

    if((sock = cddb_http_open_connection(server, 80)) == 0)
        return FALSE;

    readstr = g_strdup_printf(
            "GET /~cddb/cddb.cgi?cmd=cddb+read+%s+%08x%s&proto=%d HTTP/1.0\r\n\r\n",
            cddb_info->category, cddb_info->discid, cddb_generate_hello_string(),
            proto);

    write(sock, readstr, strlen(readstr));
    g_free(readstr);

    if (http_read_first_line(sock, buffer, 256) < 0)
      {
        http_close_connection(sock);
        return FALSE;
      }

    //printf("Read response: %s\n", buffer);

    do 
      {
        char *bufcpy,*tok1, *tok2;
        
        /* find out which element this is */
        bufcpy = strdup(buffer);
        
        if (strchr(bufcpy,'='))
          {
            tok1 = strtok_r(bufcpy,"=",&bufcpy);
            tok2 = strtok_r(NULL,"=",&bufcpy);
          }
        else continue;
        

        //fprintf(stderr,"%s %s\n",tok1,tok2);
        
        if (!strcmp(tok1,"DTITLE"))
            cdda_cdinfo_cd_set(cdinfo, tok2);
        else if (strstr(tok1, "TTITLE"))
          {
            num = atoi(tok1+6);
            cdda_cdinfo_track_set(cdinfo, num, NULL, tok2);
            //printf("%d %s\n", cdinfo->tracks[num].num, cdinfo->tracks[num].title);
          }

    } while (http_read_line(sock, buffer, 256) >= 0);

    http_close_connection(sock);
    return TRUE;
}

// gtk stuff for dialog
//int create_inexact_match_window (struct cddb_query query);

       
#define NOARG
#define BUFFERSIZE 254
#ifdef DEVELOP
static gint cddb_dialog_selection;

// ----------------------------
void
on_list1_select_row                   (GtkCList        *list,
                                        gint             row,  
                                        gint             column,
                                        GdkEvent        *event, 
                                        gpointer         user_data)
{
	user_data = &row;
}


void
on_inexact_match_button_ok_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{

}

// ----------------------------
#endif


static gint cddb_check_protocol_level(gchar *server)
{
	int level = 0, sock, n;
	char *str, buffer[256];

	if((sock = cddb_http_open_connection(server, 80)) == 0)
		return 0;
		
	str = g_strdup_printf(
		"GET /~cddb/cddb.cgi?cmd=stat%s&proto=1 HTTP/1.0\r\n\r\n",
		cddb_generate_hello_string());
		
	write(sock, str, strlen(str));
	g_free(str);
	
	if ((n = http_read_first_line(sock, buffer, 256)) < 0 ||
	    atoi(buffer) != 210)
	{
		if (n > 0)
			printf(_("Getting cddb protocol level failed: %s"),
				 buffer);
		else
			printf(_("Getting cddb protocol level failed."));
			
		http_close_connection(sock);
		return 0;
	}

	while (http_read_line(sock, buffer, 256) >= 0)
	{
		g_strstrip(buffer);
		if (!strncmp(buffer, "max proto:", 10))
			level = atoi(buffer + 10);
		if (!strcmp(buffer, "."))
			break;
	}
	http_close_connection(sock);
	//printf("\nGetting cddb protocol level. Got level %d\n", level);

        if (level < CDDB_MAX_PROTOCOL_LEVEL) return level;
        else return CDDB_MAX_PROTOCOL_LEVEL;
}

struct cdinfo_t *cddb_lookup(char *devicename, char *cddbserver)
{
    int proto,i;
    gboolean status;
    cdinfo_t *cdinfo;
    struct cddb_toc_entry *cte;
    struct cddb_disc_header_t cddb_disc_info;

    cdinfo = malloc( sizeof(cdinfo_t) );

    cte = cddb_tocentry( devicename );
    if (!cte) return;

    proto = cddb_check_protocol_level(cddbserver);

    if (proto == 0)
      {
        errorfunc(_("I did'nt get a valid protocol from the CDBB server"));
        return NULL;
      }

    status = cddb_query(cddbserver, cte, proto, &cddb_disc_info);
    if (!status)
      {
        errorfunc(_("The CDDB query failed"));
        return NULL;
      }

    cdinfo->numtrack = cte->numtrack;
    status = cddb_read(cddbserver, proto, &cddb_disc_info, cdinfo);
    if (!status)
      {
        errorfunc(_("There was an error while reading the CDDB data"));
        return NULL;
      }

    return(cdinfo);
}
