/*
  Code for getting the CDDB disc id was taken more or less verbatim from
  Robert Woodcock's cd-discid.c
  Copyright (C) Robert Woodcock
  Licensed under GPL
*/

#include "cddb.h"
#include "mcallbacks.h"
#include "minterface.h"

/* a helper function create the error dialog and set the label */
static void errorfunc(char *msg)
{
    GtkWidget *w;
    GladeXML *xml = create_error_dialog();

    w = glade_xml_get_widget(xml, "error_dialog_label");
    gtk_label_set_text(GTK_LABEL(w), msg);
}

/**********************************************************
 *
 * CDDB Disc ID stuff
 *
 **********************************************************/
int cddb_sum (int n)
{
	/* a number like 2344 becomes 2+3+4+4 (13) */
	int ret=0;

	while (n > 0) {
		ret = ret + (n % 10);
		n = n / 10;
	}

	return ret;
}

struct cddb_toc_entry *cddb_tocentry(char *devicename)
{
    int len, drive, i, totaltime;
    long int cksum = 0;
    unsigned char first=1, last=1;
    struct cdrom_tochdr hdr;
    struct cdrom_tocentry *TocEntry;
    
    struct cddb_toc_entry *retval;
    
    retval = malloc( sizeof(struct cddb_toc_entry)  * 1);

#if defined(__OpenBSD__) || defined(__NetBSD__)
    struct ioc_read_toc_entry t;
#endif

    drive = open(devicename, O_RDONLY | O_NONBLOCK);
    if (drive < 0)
      {
        char *errstr = strerror(errno);
        char *msg = malloc( sizeof(char) *
                ( strlen(errstr) +
                  strlen(devicename) + 4 ));
        sprintf(msg,"%s : %s", devicename,errstr);
        errorfunc(msg);
        free(msg);
        return(NULL);
      }

    if (ioctl(drive, CDROMREADTOCHDR, &hdr) < 0)
      {
        /* CDROMREADTOCHDR */
        char *errstr = strerror(errno);
        char *msg = malloc( sizeof(char) *
                ( strlen(errstr) +
                  strlen(devicename) + 4 ));
        sprintf(msg,"%s : %s", devicename,errstr);
        errorfunc(msg);
        free(msg);
        return(NULL);
      }

    first = hdr.cdth_trk0;
    last = hdr.cdth_trk1;

    len = (last + 1) * sizeof(struct cdrom_tocentry);

    TocEntry = malloc(len);
    if (!TocEntry) 
      {
        char *msg = malloc( sizeof(char) *
                ( strlen("cd-discid: : Can't allocate memory for TOC entries") +
                  strlen(devicename) + 1 ));
        sprintf(msg,"cd-discid: %s: Can't allocate memory for TOC entries", devicename);
        errorfunc(msg);
        free(msg);
        return(NULL);
      }
#if defined(__OpenBSD__) 
    t.starting_track = 0;
#elif defined(__NetBSD__)
    t.starting_track = 1;
#endif

#if defined(__OpenBSD__) || defined(__NetBSD__)
    t.address_format = CDROM_LBA;
    t.data_len = len;
    t.data = TocEntry;
    /* Need to find out why/if NetBSD needs this -- rcw */
    memset(TocEntry, 0, len);

    if (ioctl(drive, CDIOREADTOCENTRYS, (char *) &t) < 0)
      {
        fprintf(stderr, "cd-discid: %s: ", argv[1]);
        perror("CDIOREADTOCENTRYS");
      }

#else /* FreeBSD, Linux, Solaris */
    for (i=0; i < last; i++)
      {
        /* tracks start with 1, but I must start with 0 on OpenBSD */
        TocEntry[i].cdte_track = i + 1;
        TocEntry[i].cdte_format = CDROM_LBA;
        if (ioctl(drive, CDROMREADTOCENTRY, &TocEntry[i]) < 0)
          {
            /* CDROMREADTOCENTRY */
            char *msg, *errstr;
            errstr = strerror(errno);
            msg = malloc( sizeof(char*) *
                    ( strlen(errstr) + 
                      strlen(devicename) + 4));
            sprintf(msg,"%s : %s",devicename, errstr);
            errorfunc(msg);
            free(msg);
            return(NULL);
          }
      }

    TocEntry[last].cdte_track = CDROM_LEADOUT;
    TocEntry[last].cdte_format = CDROM_LBA;
    if (ioctl(drive, CDROMREADTOCENTRY, &TocEntry[i]) < 0)
      {
        /* CDROMREADTOCENTRY */
        char *msg, *errstr;
        errstr = strerror(errno);
        msg = malloc( sizeof(char*) *
                ( strlen(errstr) + 
                  strlen(devicename) + 4));
        sprintf(msg,"%s : %s",devicename, errstr);
        errorfunc(msg);
        free(msg);
        return(NULL);
      }
#endif

#if defined(__FreeBSD__)
    TocEntry[i].cdte_track_address = ntohl(TocEntry[i].cdte_track_address);
#endif       

    for (i=0; i < last; i++)
      {
#if defined(__FreeBSD__)
        TocEntry[i].cdte_track_address = ntohl(TocEntry[i].cdte_track_address);
#endif
        cksum += cddb_sum((TocEntry[i].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);
      }

    totaltime = ((TocEntry[last].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES) -
        ((TocEntry[0].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);

    /* print discid */
    //printf("%08lx", (cksum % 0xff) << 24 | totaltime << 8 | last);

    /* print number of tracks */
    //printf(" %d", last);

    /* print frame offsets of all tracks */
    //for (i = 0; i < last; i++)
    //  {
    //    printf(" %d", TocEntry[i].cdte_track_address + CD_MSF_OFFSET);
    //  }

    /* print length of disc in seconds */
    //printf(" %d\n", (TocEntry[last].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);

    retval->discid = (cksum % 0xff) << 24 | totaltime << 8 | last;
    retval->length = (TocEntry[last].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES;
    retval->numtrack = last;
    retval->offsets = malloc( sizeof(int) * (last+1) );
    for (i = 0; i < last; i++)
        retval->offsets[i] = TocEntry[i].cdte_track_address + CD_MSF_OFFSET;

    free(TocEntry);

    return( retval );
}

